"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const discord_js_1 = require("discord.js");
const path_1 = require("path");
const loader_1 = __importDefault(require("../utils/loader"));
class CommandHandler extends discord_js_1.Collection {
    constructor(client, dir) {
        super();
        this.setup(client, dir);
        this.client = client;
    }
    setup(client, dir) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const [filePath, fileName] of loader_1.default(path_1.join(dir, 'commands'))) {
                yield this.registerCommand(client, filePath, fileName);
            }
        });
    }
    registerCommand(client, filePath, fileName) {
        const _super = Object.create(null, {
            set: { get: () => super.set }
        });
        return __awaiter(this, void 0, void 0, function* () {
            let Command = yield Promise.resolve().then(() => __importStar(require(filePath)));
            if (Command.default && Object.keys(Command).length === 1)
                Command = Command.default;
            Command = new Command();
            const { name, callback } = Command;
            if (!callback)
                throw new Error(`No callback found on "${fileName}".`);
            console.log('Loading command: ', fileName);
            _super.set.call(this, name !== null && name !== void 0 ? name : fileName, Command);
        });
    }
    unRegisterCommand(name) {
        return super.delete(name);
    }
    performCommand(message) {
        const filteredMessage = this.filterMessage(this.client.prefix, message);
        if (!filteredMessage)
            return;
        const { command } = filteredMessage;
        try {
            const response = command.callback({ client: this.client, message, args: [] });
            if (!response)
                return;
            if (typeof response === 'string') {
                message.reply({ content: response });
            }
            else if (typeof response === 'object') {
                if (response.custom) {
                    message.reply(response);
                }
                else {
                    const embeds = Array.isArray(response) ? response : [response];
                    message.reply({ embeds });
                }
            }
        }
        catch (e) {
            console.log(`Couldn't perform command ${message.content}.`);
        }
    }
    filterMessage(prefix, message) {
        let content = message.content;
        if (!content.toLowerCase().startsWith(prefix.toLowerCase()))
            return;
        if (message.author.bot)
            return;
        content = content.substring(prefix.length);
        const args = content.split(/[ ]+/g);
        const first = args.shift();
        if (!first)
            return;
        const commandName = first.toLowerCase();
        const command = this.get(commandName);
        if (!command)
            return;
        return {
            prefix: prefix,
            name: commandName,
            command: command
        };
    }
}
exports.default = CommandHandler;
